--
-- Mod: Animals Keeper
--
-- @Author: D3r1L
-- @Website: 
-- @Email: 
-- @Date: 22.08.2019
-- @Version: 1.0.0.5
-- Copyright (C) D3r1L, All Rights Reserved.

animalsKeeper = {};

local modDesc = loadXMLFile("modDesc", g_currentModDirectory .. "modDesc.xml")

animalsKeeper.version = getXMLString(modDesc, "modDesc.version")
animalsKeeper.modDirectory = g_currentModDirectory

animalsKeeper.costs = {5.00, 10.00, 15.00, 20.00, 25.00, 30.00, 35.00, 40.00, 45.00, 50.00}
animalsKeeper.stables = {0, 1, 2, 3, 4}
animalsKeeper.speed = {1, 2, 3, 4, 5}

-- Global values for settings
animalsKeeper.animalsFillStraw = 1. --true
animalsKeeper.animalStrawCost = 30.00

animalsKeeper.animalsFillWater = true
animalsKeeper.animalWaterCost = 30.00

animalsKeeper.animalsCleanliness = true
animalsKeeper.animalCleanCost = 30.00

animalsKeeper.speedPlayer = 1
animalsKeeper.speedPlayerBkp = 1
animalsKeeper.speedPlayerRun = 1

-- Settings
animalsKeeper.settingsDir = ""
animalsKeeper.savegameFold = ""

-- Hour paid
animalsKeeper.hourPaid = -1

addModEventListener(animalsKeeper)

function animalsKeeper:init()
end

function animalsKeeper:loadMap()	

	--print("###*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*##")
	--print("##                Animals Keeper v " .. self.version .. "		   ##")
	--print("##		      by D3r1L				   ##")
	--print("###*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*##")
	
	--DebugUtil.printTableRecursively(g_currentMission, "-", 0, 5)
	
	local savegameDir;
	--local savegameFold;
	local savegameFolder;
	if g_currentMission.missionInfo.savegameDirectory then
		savegameDir = g_currentMission.missionInfo.savegameDirectory;
	end
	if not savegameDir and g_careerScreen.currentSavegame and g_careerScreen.currentSavegame.savegameIndex then
		savegameDir = ('%ssavegame%d'):format(getUserProfileAppPath(), g_careerScreen.currentSavegame.savegameIndex);
	end
	if not savegameDir and g_currentMission.missionInfo.savegameIndex ~= nil then
		savegameDir = ('%ssavegame%d'):format(getUserProfileAppPath(), g_careerScreen.missionInfo.savegameIndex);
	end
	for savegameFolder in string.gmatch(savegameDir, '([^/]+)') do
		--print("--------------- animalsKeeper.savegameDir: " .. savegameFolder)
		animalsKeeper.savegameFold = savegameFolder
	end
	
	-- Settings
	animalsKeeper.settingsDir = getUserProfileAppPath().. "modsSettings/animalKeeper/"
	
	-- Init data tables
	animalsKeeper.eventName = {}

	-- Gui Class inheritance
	animalsKeeper.gui = {}
	animalsKeeper.gui["awSettingGui"] = awGui:new()
	g_gui:loadGui(animalsKeeper.modDirectory .. "gui/awGui.xml", "awGui", animalsKeeper.gui.awSettingGui)

	-- Load Mod Settings
	animalsKeeper:LoadSettings()
	--animalsKeeper:SaveSettings()
	animalsKeeper:updateGui()
	
	-- Player action event registration
	Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, animalsKeeper.registerActionEventsPlayer)
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, animalsKeeper.removeActionEventsPlayer)
	
	-- Speed Player
	Mission00.onStartMission = Utils.appendedFunction(Mission00.onStartMission, animalsKeeper.afterLoad);
end

function animalsKeeper:afterLoad()
    animalsKeeper.speedPlayerBkp = g_currentMission.player.motionInformation.maxWalkingSpeed
	animalsKeeper.speedPlayerRun = g_currentMission.player.motionInformation.maxRunningSpeed
	
	g_currentMission.player.motionInformation.maxWalkingSpeed = (animalsKeeper.speedPlayerBkp * animalsKeeper.speedPlayer)
	g_currentMission.player.motionInformation.maxRunningSpeed = (animalsKeeper.speedPlayerRun * animalsKeeper.speedPlayer)
	
	print("AnimalsKeeper.SpeedPlayer = " .. tostring(animalsKeeper.speedPlayer));
end

function animalsKeeper:LoadSettings()
	
	local file = animalsKeeper.settingsDir .. animalsKeeper.savegameFold .. "_animalsKeeper.xml"
	
	local pos = 0
	local rootPath = "animalsKeeper.settings"
	local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)
	
	if fileExists(file) then
		---- Load
		local xmlContent = loadXMLFile("animalsKeeper_XML", file, "animalsKeeper")
		
		--animalsFillStraw 	= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#animalsFillStraw"), true)
		animalsKeeper.animalsFillStraw = Utils.getNoNil(getXMLInt(xmlContent, groupNameTag .. string.format("#animalsFillStraw")), animalsKeeper.animalsFillStraw)
		animalsKeeper.animalStrawCost = Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#animalStrawCost"), animalsKeeper.animalStrawCost)
		
		animalsKeeper.animalsFillWater = Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#animalsFillWater"), true)
		animalsKeeper.animalWaterCost = Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#animalWaterCost"), 30.00)
		
		animalsKeeper.animalsCleanliness	= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#animalsCleanliness"), true)
		animalsKeeper.animalCleanCost = Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#animalCleanCost"), 30.00)
		
		animalsKeeper.speedPlayer = Utils.getNoNil(getXMLInt(xmlContent, groupNameTag .. string.format("#speedPlayer")), animalsKeeper.speedPlayer)		
	else
		createFolder(getUserProfileAppPath() .. "modsSettings/")
		createFolder(animalsKeeper.settingsDir)
		
		---- Create
		local xmlContent = createXMLFile("animalsKeeper_XML", file, "animalsKeeper")
		
		--setXMLBool(xmlContent, groupNameTag .. "#animalsFillStraw", animalsFillStraw)
		setXMLInt(xmlContent, groupNameTag .. string.format("#animalsFillStraw"), animalsKeeper.animalsFillStraw)
		setXMLFloat(xmlContent, groupNameTag .. "#animalStrawCost", animalsKeeper.animalStrawCost)
		
		setXMLBool(xmlContent, groupNameTag .. "#animalsFillWater", animalsKeeper.animalsFillWater)
		setXMLFloat(xmlContent, groupNameTag .. "#animalWaterCost", animalsKeeper.animalWaterCost)
		
		setXMLBool(xmlContent, groupNameTag .. "#animalsCleanliness", animalsKeeper.animalsCleanliness)
		setXMLFloat(xmlContent, groupNameTag .. "#animalCleanCost", animalsKeeper.animalCleanCost)
		
		setXMLInt(xmlContent, groupNameTag .. string.format("#speedPlayer"), animalsKeeper.speedPlayer)
		
		saveXMLFile(xmlContent)
		--delete(xmlContent)
	end
end

function animalsKeeper:SaveSettings()
	
	createFolder(getUserProfileAppPath() .. "modsSettings/")
	createFolder(animalsKeeper.settingsDir)
	
	local file = animalsKeeper.settingsDir .. animalsKeeper.savegameFold .. "_animalsKeeper.xml"
	local pos = 0
	local rootPath = "animalsKeeper.settings"
	local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)
	
	local xmlContent = createXMLFile("animalsKeeper_XML", file, "animalsKeeper")
	
	--setXMLBool(xmlContent, groupNameTag .. "#animalsFillStraw", animalsFillStraw)
	setXMLInt(xmlContent, groupNameTag .. string.format("#animalsFillStraw"), animalsKeeper.animalsFillStraw)
	setXMLFloat(xmlContent, groupNameTag .. "#animalStrawCost", animalsKeeper.animalStrawCost)
	
	setXMLBool(xmlContent, groupNameTag .. "#animalsFillWater", animalsKeeper.animalsFillWater)
	setXMLFloat(xmlContent, groupNameTag .. "#animalWaterCost", animalsKeeper.animalWaterCost)
	
	setXMLBool(xmlContent, groupNameTag .. "#animalsCleanliness", animalsKeeper.animalsCleanliness)
	setXMLFloat(xmlContent, groupNameTag .. "#animalCleanCost", animalsKeeper.animalCleanCost)
	
	setXMLInt(xmlContent, groupNameTag .. string.format("#speedPlayer"), animalsKeeper.speedPlayer)
	
	saveXMLFile(xmlContent)
	--delete(xmlContent)
	
end

function animalsKeeper:delete()
end

function animalsKeeper:registerActionEventsPlayer()
	-- Animals Keeper Settings
	local r, eventName = InputBinding.registerActionEvent(g_inputBinding, 'animalsKeeper_Setting', self, animalsKeeper.actionAnimalKeeper_Setting, false, true, false, true)
	if r then
		table.insert(animalsKeeper.eventName, eventName)
		g_inputBinding.events[eventName].displayIsVisible = true
		g_inputBinding:setActionEventText(eventName, g_i18n:getText("input_animalsKeeper_Setting"))
	end
end

function animalsKeeper:removeActionEventsPlayer()
	animalsKeeper.eventName = {}
end

-- Show settings menu
function animalsKeeper:actionAnimalKeeper_Setting(actionName, keyStatus, arg3, arg4, arg5)
	if animalsKeeper.gui.awSettingGui.isOpen then
		animalsKeeper.gui.awSettingGui:onClickBack()
	elseif g_gui.currentGui == nil then
		g_gui:showGui("awGui")
	end
end

function animalsKeeper:keyEvent(unicode, sym, modifier, isDown)
end

function animalsKeeper:mouseEvent(posX, posY, isDown, isUp, button)
end

function animalsKeeper:deleteMap()
end

function animalsKeeper:updateGui()
	
	-- Keeper Fill Straw
	--animalsKeeper.gui.awSettingGui:setStraw(animalsFillStraw)
	index = 0
	for i = 1, #animalsKeeper.stables, 1 do
		if (animalsKeeper.stables[i] == animalsKeeper.animalsFillStraw) then
			index = i
			break
		end
	end
	--print("index = " .. tostring(index) .. " | animalsFillStraw = " .. tostring(animalsKeeper.animalsFillStraw))
	--if (index ~= 0) then
		animalsKeeper.gui.awSettingGui:setStrawStable(index)
	--end
	
	-- Keeper Fill Straw Cost
	index = 0
	for i = 1, #animalsKeeper.costs, 1 do
		if (animalsKeeper.costs[i] == animalsKeeper.animalStrawCost) then
			index = i
			break
		end
	end
	if (index ~= 0) then
		animalsKeeper.gui.awSettingGui:setStrawCost(index)
	end
	
	-- Keeper Fill Water
	animalsKeeper.gui.awSettingGui:setWater(animalsKeeper.animalsFillWater)
	
	-- Keeper Fill Water Cost
	index = 0
	for i = 1, #animalsKeeper.costs, 1 do
		if (animalsKeeper.costs[i] == animalsKeeper.animalWaterCost) then
			index = i
			break
		end
	end
	if (index ~= 0) then
		animalsKeeper.gui.awSettingGui:setWaterCost(index)
	end
	
	-- Keeper Cleaning Pens
	animalsKeeper.gui.awSettingGui:setCleanPens(animalsKeeper.animalsCleanliness)
	
	-- Keper Cleaning Cost
	index = 0
	for i = 1, #animalsKeeper.costs, 1 do
		if (animalsKeeper.costs[i] == animalsKeeper.animalCleanCost) then
			index = i
			break
		end
	end
	if (index ~= 0) then
		animalsKeeper.gui.awSettingGui:setCleanPensCost(index)
	end
	
	-- SpeedPlayer
	index = 0
	for i = 1, #animalsKeeper.stables, 1 do
		if (animalsKeeper.speed[i] == animalsKeeper.speedPlayer) then
			index = i
			break
		end
	end
	--print("index = " .. tostring(index) .. " | animalsFillStraw = " .. tostring(animalsKeeper.animalsFillStraw))
	--if (index ~= 0) then
	animalsKeeper.gui.awSettingGui:setSpeedPlayer(index)
end

function animalsKeeper:settingsFromGui(sStraw, sStrawCost, sWaterCost, sWater, sPens, sPensCost, sSpeed)
	-- Update settings
	--animalsFillStraw = sStraw
	animalsKeeper.animalsFillStraw = animalsKeeper.stables[sStraw]
	animalsKeeper.animalStrawCost = animalsKeeper.costs[sStrawCost]
	
	animalsKeeper.animalsFillWater = sWater
	animalsKeeper.animalWaterCost = animalsKeeper.costs[sWaterCost]
	
	animalsKeeper.animalsCleanliness = sPens
	animalsKeeper.animalCleanCost = animalsKeeper.costs[sPensCost]
	
	-- Speed Player
	animalsKeeper.speedPlayer = animalsKeeper.speed[sSpeed]
	if g_currentMission.controlPlayer then
		--animalsKeeper.speedPlayerBkp = g_currentMission.player.motionInformation.maxWalkingSpeed
		--animalsKeeper.speedPlayerRun = g_currentMission.player.motionInformation.maxRunningSpeed
		
		g_currentMission.player.motionInformation.maxWalkingSpeed = (animalsKeeper.speedPlayerBkp * animalsKeeper.speedPlayer)
		g_currentMission.player.motionInformation.maxRunningSpeed = (animalsKeeper.speedPlayerRun * animalsKeeper.speedPlayer)
		
		print("AnimalsKeeper.SpeedPlayer = " .. tostring(animalsKeeper.speedPlayer));
		--[[
			if animalsKeeper.speedPlayer == 1 then
				print("AnimalsKeeper.SpeedPlayer -- Player normal speed");
				g_currentMission.player.motionInformation.maxWalkingSpeed = 4;		
				--g_currentMission.player.motionInformation.maxRunningSpeed = 9;		
			end;
			
			if animalsKeeper.speedPlayer == 2 then
				print("AnimalsKeeper.SpeedPlayer -- Player speed increase x2");
				g_currentMission.player.motionInformation.maxWalkingSpeed = 8;
				--g_currentMission.player.motionInformation.maxRunningSpeed = 18;
			end;
			
			if animalsKeeper.speedPlayer == 4 then
				print("AnimalsKeeper.SpeedPlayer -- Player speed increase x4");
				g_currentMission.player.motionInformation.maxWalkingSpeed = 16;		
				--g_currentMission.player.motionInformation.maxRunningSpeed = 36;
			end;
			
			if animalsKeeper.speedPlayer == 6 then
				print("AnimalsKeeper.SpeedPlayer -- Player speed increase x6");
				g_currentMission.player.motionInformation.maxWalkingSpeed = 24;		
				--g_currentMission.player.motionInformation.maxRunningSpeed = 54;
			end;
			
			if animalsKeeper.speedPlayer == 8 then
				print("AnimalsKeeper.SpeedPlayer -- Player speed increase x8");
				g_currentMission.player.motionInformation.maxWalkingSpeed = 32;		
				--g_currentMission.player.motionInformation.maxRunningSpeed = 72;
			end;
			--g_currentMission.player.motionInformation.maxWalkingSpeed = g_currentMission.player.motionInformation.maxWalkingSpeed * animalsKeeper.speedPlayer
			--g_currentMission.player.motionInformation.maxRunningSpeed = g_currentMission.player.motionInformation.maxRunningSpeed * animalsKeeper.speedPlayer
		--]]
	end
	
	-- Save to xml
	animalsKeeper:SaveSettings()
	--animalsKeeper:LoadSettings()
end

function animalsKeeper:update(dt)
	
	local hourNow = g_currentMission.environment.currentHour
	local minuteNow = g_currentMission.environment.currentMinute
	local numBeast = 0
	local price = 0
	
	local playerOwnsFarm = false
	
	if animalsKeeper.hourPaid ~= hourNow then
		animalsKeeper.hourPaid = hourNow
	-- Keeper hour price
	--if minuteNow ~= 00 then
	--	self.doHour = false
	--end
	--if minuteNow == 00 and not self.doHour then
		-- Every Hour
	--	self.doHour = true
		
		farmData = {
			ownerFarmId = 0,
			farmId = 0,
		}
		
		farmData.ownerFarmId = g_currentMission.player.ownerFarmId
		farmData.farmId = g_currentMission.player.farmId
		
		function checkIsOwnerOfFarm(husbandry)
			if g_currentMission:getIsClient() then
				farmData.ownerFarmId = g_currentMission.player.ownerFarmId
				farmData.farmId = g_currentMission.player.farmId
				
				if husbandry.modulesByName.animals.owner.ownerFarmId == farmData.farmId then
					playerOwnsFarm = true
					return true
				else
					playerOwnsFarm = false
					return false
				end
			end
		end
		
		function animalCount(husbandry)
			if husbandry ~= nil then
				local animals = husbandry.modulesByName.animals.animals
				local numAnimals = 0
				for k,v in pairs(animals) do
					numAnimals = tonumber(k)
				end
				return numAnimals
			else
				print("bad parameters for animalCount()")
				return 0
			end
		end
		
		local putStraw = false
		animalsKeeper.cleanlinessFactor = 0
		
		for k,husbandry in pairs(g_currentMission.husbandries) do
			numBeast = animalCount(husbandry)
			--print("numBeast: " .. tostring(numBeast))
			if checkIsOwnerOfFarm(husbandry) and numBeast > 0 then
				husbandry.modulesByName.foodSpillage.hasCleanliness = true
				-- Cleanliness
				--print("animalsCleanliness: " .. tostring(animalsKeeper.animalsCleanliness))
				--print("foodSpillage: " .. tostring(husbandry.modulesByName.foodSpillage))
				--print("cleanlinessFactor: " .. tostring(husbandry.modulesByName.foodSpillage.cleanlinessFactor))
				if husbandry.modulesByName.foodSpillage ~= nil and animalsKeeper.animalsCleanliness then
					husbandry.modulesByName.foodSpillage.hasCleanliness = false
					husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
					husbandry:setModuleParameters("foodSpillage", 0, 0)
					price = price + (numBeast * animalsKeeper.animalCleanCost)
					--print("animalCleanCost: " .. animalCleanCost)
				end
				
				-- Water
				if husbandry.modulesByName.water ~= nil and animalsKeeper.animalsFillWater then
					capacity = husbandry.modulesByName.water.fillCapacity
					husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)
					price = price + (numBeast * animalsKeeper.animalWaterCost)
					--print("animalWaterCost: " .. animalWaterCost)
				end
				
				-- Straw
				if husbandry.modulesByName.straw ~= nil and animalsKeeper.animalsFillStraw > 0 then
					if animalsKeeper.animalsFillStraw == 1 then
						putStraw = true
					elseif animalsKeeper.animalsFillStraw == 2 and husbandry.modulesByName.animals.animalType ~= "COW" then
						putStraw = true
					elseif animalsKeeper.animalsFillStraw == 3 and husbandry.modulesByName.animals.animalType ~= "PIG" then
						putStraw = true
					elseif animalsKeeper.animalsFillStraw == 4 and husbandry.modulesByName.animals.animalType ~= "HORSE" then
						putStraw = true
					end
					
					if putStraw then
						capacity = husbandry.modulesByName.straw.fillCapacity
						husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, capacity)
						price = price + (numBeast * animalsKeeper.animalStrawCost)
						--print("animalStrawCost: " .. animalStrawCost)
					end
					putStraw = false
				end
			end
		end
		
		--print("hourNow: " .. hourNow .. " | minuteNow: " .. minuteNow .. " - price: " .. price)
		
		-- Keeper payment every Hour
		if price > 0 then
			local pricePerHour = price / 24
			
			--print("price = " .. price .. " | pricePerHour: " .. pricePerHour)
			
			g_currentMission:farmStats(farmData.farmId):updateStats("expenses", pricePerHour)
			g_currentMission:addMoney(-pricePerHour, farmData.farmId, MoneyType.ANIMAL_UPKEEP, true)
		end
		--[[
		-- Keeper total ammount at MIDNIGHT
		if hourNow == 00 and minuteNow == 00 and price > 0 then
			--print("price OK: " .. price)
			
			local moneyCurrency = " $"
			if g_gameSettings.moneyUnit == 1 then
				moneyCurrency = " €" 
			elseif g_gameSettings.moneyUnit == 2 then
				moneyCurrency = " $"
			elseif g_gameSettings.moneyUnit == 3 then
				moneyCurrency = " £"
			end;
			
			local priceFormat = tostring(g_i18n:formatNumber(price, 0))
			--local showKeeperPay = g_inputBinding:setActionEventText(eventName, g_i18n:getText("show_Keeper_pay"))
			local showKeeperPay = g_i18n:getText("show_Keeper_pay")
			
			--g_currentMission:farmStats(farmData.farmId):updateStats("expenses", price)
			--g_currentMission:addMoney(-price, farmData.farmId, MoneyType.ANIMAL_UPKEEP, true)
			g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, showKeeperPay .. " (- " .. priceFormat .. " " .. moneyCurrency .. ")");
		end--]]
	end
end